#!/bin/ksh93
#set tabstop=4 to display correctly in vi
#
#-------------------------------------------------------------------------------
# File Name:    DropInoperativeViews.ksh (ksh93)
#				Based on existing copyleft code at website http://www.sustainablesoftware.net - please respect these terms.
# Author:       Alex Levy t/a http://www.sustainablesoftware.net
#
# Purpose:      Drop all views marked as inoperative or invalid and that are: 
#				a. older than $3 days
#				b. user views, not system views!
#               Export the view defnitions before dropping.
#              
# Out of scope: Inoperative MQTs
# 
# Usage:        DropInoperativeViews.ksh $1 instance_name $2 database_name $3 age in days after which inoperative views are dropped
#				To drop all inoperative views regardless, use a value for $3 <= 0.
#               Assumes runner is the instance owner.
#
# File dependencies:
#               ${INSTHOME}/sqllib/db2profile
#               ${INSTHOME}/sqllib/userprofile 
#               ${INSTHOME}/FUNCTIONS   (the userprofile exports $FPATH).
#				${INSTHOME}/MAINTENANCE (the userprofile exports $MAINTENANCE_DIR).
#				${INSTHOME}/MAINTENANCE/log  (the userprofile exports $MAINTENANCE_LOGS).
#				${INSTHOME}/MAINTENANCE/log/work (the userprofile exports $MAINTENANCE_WORK).
#				${MAINTENANCE_DIR}/TEMPLATES/FindInoperativeViews.template   (SQL templates)
#				${MAINTENANCE_DIR}/TEMPLATES/ExportInoperativeViews.template 
#
# Calls:		${MAINTENANCE}/Exit.ksh on error	
#
# File dependencies for partitioned databases:
#               None - operates against the system catalog partition only. This is assumed to have DB2NODE=0
#
# Notes:		Note that inoperative views are never usable after the base table is dropped and recreated. 
# 				For one reason, the drop of the base table removes the record of the dependency from SYSCAT.TABDEP.
# 				Therefore we only need generate drop statements from a read of SYSCAT.TABLES.
# 				The only point in preserving such views less than $3 days old is to identify untidy processes.
# Date:         April 2012.           
#-------------------------------------------------------------------------------

# Functions in $FPATH never exit, they only return.
# We can either test each return code or use a trap. If we do use a trap, and we wish to test the return code first,
# then the test must be part of the same statement, e.g. F_CONNECT $DBNAME $DEBUG_MODE || die.
# The trap statement and the 2 inline functions 'FatalError' and 'die' can handle the exit for all error conditions.
# The 'die' function is called on any error detected by the program logic.
# The FatalError function is triggered by any non-zero signal, including the exit code from 'die'.

trap FatalError ERR

function FatalError {
# Single point of exit on any error condition.
# This relies on a signal trap. ERR is any non-zero exit status and is Korn shell only. 
# See 'man trap' for further detail.
# Can also call this function by 'kill -s ERR $$ 2>/dev/null'

RC=$? 
if   (( RC == 99 ))
then # error has already been trapped by function 'die', do nothing more except exit
	 ${MAINTENANCE_DIR}/Exit.ksh 99	
else
     print ERR signal received
	 if   [[ $LOG == "NOLOG" || ! -f $LOG || ! -s $LOG ]]
	 then F_MESSAGE "$script Ends with error(s), see stdout above." PLAIN I +x
	 else F_RESET_OUTPUT
     	  cat $LOG
     	  F_MESSAGE "$script ends with error(s). Please review log file $LOG . " PLAIN I +x | tee -a $LOG
     	  F_MESSAGE "The log file is displayed above for ease of reference. " PLAIN I +x | tee -a $LOG
	 fi
	 ${MAINTENANCE_DIR}/Exit.ksh 4	
fi
}

function die {
# Single point of exit on any error condition caught by the program logic.
# Restore stdout and stderr if required, before flowing to FatalError.

LOG=${LOG:=NOLOG} # use $LOG if set, otherwise assign a value of NOLOG to $LOG.
LINE=${1:=unknown}
LOCATION=${2:=unknown}

if   [[ $LOG == "NOLOG" || ! -f $LOG || ! -s $LOG ]]
then F_MESSAGE "$script Ends with error(s) at line $LINE, location $LOCATION; see stdout above." PLAIN I +x
else F_RESET_OUTPUT
     cat $LOG
     F_MESSAGE "$script ends with error(s). Please review log file $LOG . " PLAIN I +x | tee -a $LOG
     F_MESSAGE "The log file is displayed above for ease of reference. " PLAIN I +x | tee -a $LOG
fi

# trigger FatalError
exit 99
} 

#-------------------------------------------------------------------------------
# Declare and test variables.
#-------------------------------------------------------------------------------

set -u								# assert all variables are defined and set.	 
typeset script=$(basename $0 .ksh)  # Strip off path and '.ksh' extension.
DEBUG_MODE="+x"                     # Default value - disable debugging.
typeset -i RC=0						# return code from function calls and o/s
typeset -i counter=999			
now=$(date +%Y%m%d%H%M%S)              
typeset timestmp=               	# set later, holds timestamp of the oldest backup we retain.
typeset LOG=NOLOG               	# set later

# Check the call
(( $# == 3 )) || F_ERROR_MESSAGE " Usage! I said:  $0 <instance_name> <database_name> <retention_period_in_days>" $DEBUG_MODE   
(( $# == 3 )) || die $LINENO 1 
typeset -l  INSTNAME=$1
typeset -u  DBNAME=$2
typeset -i days_to_keep=$3 
if   [[ $INSTNAME == $USER ]]
then :
else
     F_ERROR_MESSAGE "$0 : The runner must be the DB2 instance owner. Exiting ..." $DEBUG_MODE
     die $LINENO 2
fi

# Assert all required files and directories are in place and are accessible or executable;
# the first function call tests for $INSTHOME and the db2profile and executes db2profile.
F_SETUP_DB2_ENVIRONMENT $INSTHOME $INSTNAME $DBNAME $DEBUG_MODE || die  $LINENO 3
F_DIR_CHECK ${INSTHOME}/deploy_db/bin $DEBUG_MODE || die  $LINENO 4
F_DIR_CHECK $MAINTENANCE_DIR  $DEBUG_MODE   || die $LINENO 6
F_DIR_CHECK $MAINTENANCE_DIR/TEMPLATES  $DEBUG_MODE   || die $LINENO 6A
F_DIR_CHECK $MAINTENANCE_LOGS $DEBUG_MODE   || die $LINENO 7
F_DIR_CHECK $MAINTENANCE_WORK $DEBUG_MODE   || die $LINENO 8 
F_READ_TEST ${MAINTENANCE_DIR}/TEMPLATES/FindInoperativeViews.template $DEBUG_MODE || die $LINENO 9
F_READ_TEST ${MAINTENANCE_DIR}/TEMPLATES/ExportInoperativeViews.template $DEBUG_MODE || die $LINENO 9A

# With these basic checks out of the way, direct all output to a timestamped log file.
LOG=${MAINTENANCE_LOGS}/${script}.${DBNAME}.${now}.log
F_INIT_OUTPUT_FILE $INSTNAME $LOG $DEBUG_MODE || die $LINENO 10
export LOG				 
F_REDIRECT_OUTPUT $LOG $DEBUG_MODE || die $LINENO 11

# check no backup is running before connecting
db2 -v attach to $INSTNAME || die $LINENO 12
while (( counter > 0 ))
do
    counter=$(db2 list utilities | grep -ic backup) || : #suppress trap of grep return code 1 if no match found
    (( counter > 0 )) && F_MESSAGE "Backup(s) are running." PLAIN I $DEBUG_MODE
    (( counter > 0 )) && sleep 300
done

sed s/DAYS_TO_KEEP/$days_to_keep/ ${MAINTENANCE_DIR}/TEMPLATES/FindInoperativeViews.template > ${MAINTENANCE_WORK}/FindInoperativeViews.sql.${now}

export DB2NODE=0 # only meaningful in DPF
F_CONNECT $DBNAME $DEBUG_MODE || die $LINENO 13
 
# Disable trap from here on, since we want explicit return code checks for SQL queries. 
# That includes RC=1 for no inoperative views found, which is not an error condition!
trap " " ERR
 
F_RUN_DB2FILE ${MAINTENANCE_WORK}/FindInoperativeViews.sql.${now} "-txf" 1 ${MAINTENANCE_WORK}/DropInoperativeViews.sql.${now} nocompress $DEBUG_MODE
RC=$?

case $RC in
	0) 	# Inoperative views found.
		# Save all current invalid/inoperative view definitions first, for possible replay. 
 		F_RUN_DB2FILE ${MAINTENANCE_DIR}/TEMPLATES/ExportInoperativeViews.template "-txf" 0 NONE nocompress $DEBUG_MODE  
		RC=$? ; (( RC > 0 )) && F_MESSAGE "View export error, return code $RC, see ${LOG}." PLAIN I $DEBUG_MODE && die $LINENO 14
		F_RUN_DB2FILE ${MAINTENANCE_WORK}/DropInoperativeViews.sql.${now} "-tvf" 0 NONE nocompress $DEBUG_MODE 
		RC=$? ; (( RC > 0 )) && F_MESSAGE "View drop error, return code $RC, see ${LOG}." PLAIN I $DEBUG_MODE && die $LINENO 15 
		;;	 
    1)  print " $0 : Good: there are no inoperative views to drop." 
		RC=0
		;;	 
	 *) F_MESSAGE "Select error against syscat.tables with return code $RC : SEE ${MAINTENANCE_WORK}/DropInoperativeViews.sql.${now} " PLAIN I $DEBUG_MODE 
		die $LINENO 16
		;; 
esac

F_TERMINATE $DEBUG_MODE  

# Restore stdout and stderr to terminal
F_RESET_OUTPUT 
cat $LOG
exit $RC
