#!/bin/ksh93
# File Name:    Get_DB_Settings.ksh (ksh93)
#				Based on existing copyleft code at website http://www.sustainablesoftware.net - please respect these terms.
# Authors:      Alex Levy t/a http://www.sustainablesoftware.net
#				Dan 'The Man' Day
# Contributor:  Steve Harris
#
# Purpose:      Record DB2 parameters, settings and configuration.
#               These include: registry variables;
#                              dbm, db, cli and das cfgs;
#                              node, database and dcs directory listings;
#                              the instance owner's crontab file;
#                              the instance owner's .profile and .rhosts;
#                              db2audit configuration;
#                              db2look, db2pd and db2level outputs;
#                              LIST TABLESPACES SHOW DETAIL command output;
#                              LIST HISTORY ARCHIVE LOGS command output;
#                              LIST HISTORY BACKUP command output;
#                              a backup of the /etc/services file;
#                              a backup of the /etc/hosts file.
#
# Usage:        Get_DB_Settings.ksh $1 $2 
#
# Parameters:   $1 = <instance_name> (e.g. db2dev)
#               $2 = <database_name> (e.g. test)
#
# Dependencies: $2 database is activated
#               ${INSTHOME}/sqllib/db2profile
#               ${INSTHOME}/sqllib/userprofile 
#               ${INSTHOME}/sqllib/db2nodes.cfg 
#               ${INSTHOME}/FUNCTIONS   (the userprofile exports $FPATH).
#				${INSTHOME}/MAINTENANCE (the userprofile exports $MAINTENANCE_DIR).
#				${INSTHOME}/MAINTENANCE/log  (the userprofile exports $MAINTENANCE_LOGS).
#				${INSTHOME}/MAINTENANCE/log/work (the userprofile exports $MAINTENANCE_WORK).
#				${INSTHOME}/sqllib/db2dump/CFG (the userprofile exports $CFG_DIR).
#
# Calls:		${MAINTENANCE}/Exit.ksh on error	
#
# File dependencies for partitioned databases:
#               The working assumption is that $INSTHOME is network-mounted on MPP DPF systems.
#
# Notes:       	Not appropriate for client or gateway instances.
#
# Last update:  September 2012.           
#-------------------------------------------------------------------------------


# Functions in $FPATH never exit, they only return.
# We can either test each return code or use a trap. If we do use a trap, and we wish to test the return code first,
# then the test must be part of the same statement, e.g. F_CONNECT $DBNAME $DEBUG_MODE || die.
# The trap statement and the 2 inline functions 'FatalError' and 'die' handle the exit for all error conditions.
# The 'die' function is called on any error detected by the program logic.
# The FatalError function is triggered by any non-zero signal, including the exit code from 'die'.

trap FatalError ERR

function FatalError {
# Single point of exit on any error condition.
# This relies on a signal trap. ERR is any non-zero exit status and is Korn shell only. 
# See 'man trap' for further detail.
# Can also call this function by 'kill -s ERR $$ 2>/dev/null'

RC=$? 
if   (( RC == 99 ))
then # error has already been trapped by function 'die', do nothing more except exit
	 ${MAINTENANCE_DIR}/Exit.ksh 99	
else
     print ERR signal received
	 if   [[ $LOG == "NOLOG" || ! -f $LOG || ! -s $LOG ]]
	 then F_MESSAGE "$script Ends with error(s), see stdout above." PLAIN I +x
	 else F_RESET_OUTPUT
     	  cat $LOG
     	  F_MESSAGE "$script ends with error(s). Please review log file $LOG . " PLAIN I +x | tee -a $LOG
     	  F_MESSAGE "The log file is displayed above for ease of reference. " PLAIN I +x | tee -a $LOG
	 fi
	 ${MAINTENANCE_DIR}/Exit.ksh 4	
fi
}

function die {
# Single point of exit on any error condition caught by the program logic.
# Restore stdout and stderr if required, before flowing to FatalError.

LOG=${LOG:=NOLOG} # use $LOG if set, otherwise assign a value of NOLOG to $LOG.
LINE=${1:=unknown}
LOCATION=${2:=unknown}

if   [[ $LOG == "NOLOG" || ! -f $LOG || ! -s $LOG ]]
then F_MESSAGE "$script Ends with error(s) at line $LINE, location $LOCATION; see stdout above." PLAIN I +x
else F_RESET_OUTPUT
     cat $LOG
     F_MESSAGE "$script ends with error(s). Please review log file $LOG . " PLAIN I +x | tee -a $LOG
     F_MESSAGE "The log file is displayed above for ease of reference. " PLAIN I +x | tee -a $LOG
fi

# trigger FatalError
exit 99
}

#-------------------------------------------------------------------------------
# Declare and test variables.
#-------------------------------------------------------------------------------

set -u								# assert all variables are defined and set.	 
typeset script=$(basename $0 .ksh)  # Strip off path and '.ksh' extension.
DEBUG_MODE="+x"                     # Default value - disable debugging.
typeset -i RC=0						# return code from function calls and o/s
typeset -i num_partitions=-1		# reset later
typeset -i count=999				
now=$(date +%Y%m%d%H%M%S)              
typeset LOG=NOLOG               	# reset later

# Check the call
(( $# == 2 )) || F_ERROR_MESSAGE " Usage! I said:  $0 <instance_name> <database_name>" $DEBUG_MODE   
(( $# == 2 )) || die $LINENO 1 
typeset -l  INSTNAME=$1
typeset -u  DBNAME=$2
typeset     INSTHOME=~${INSTNAME}

# Assert all required files and directories are in place and are accessible or executable;
# the first function call tests for $INSTHOME and the db2profile and executes db2profile.
F_SETUP_DB2_ENVIRONMENT $INSTHOME $INSTNAME $DBNAME $DEBUG_MODE || die  $LINENO 2
F_DIR_CHECK $MAINTENANCE_DIR  $DEBUG_MODE   || die $LINENO 3
F_DIR_CHECK $MAINTENANCE_LOGS $DEBUG_MODE   || die $LINENO 4
F_DIR_CHECK $MAINTENANCE_WORK $DEBUG_MODE   || die $LINENO 5 
F_DIR_CHECK $CFG_DIR          $DEBUG_MODE   || die $LINENO 6 

# check the database argument (is it catalogued as a local database?)
F_TEST_DBNAME $DBNAME $DEBUG_MODE || die $LINENO 7

# With these basic checks out of the way, direct all output to a timestamped log file.
LOG=${MAINTENANCE_LOGS}/${script}.${DBNAME}.${now}.log
F_INIT_OUTPUT_FILE $INSTNAME $LOG $DEBUG_MODE || die $LINENO 7
export LOG				 
F_REDIRECT_OUTPUT $LOG $DEBUG_MODE || die $LINENO 8

export DB2NODE=0 # only meaningful in DPF

# check no backup is running before connecting
db2 -v attach to $INSTNAME || die $LINENO 9
while (( count > 0 ))
do
    count=$(db2 list utilities | grep -ic backup | grep $DBNAME ) || : #suppress Unix trap of return code 1 if no match found
    (( count > 0 )) && F_MESSAGE "Backup(s) are running." PLAIN I $DEBUG_MODE
    (( count > 0 )) && sleep 300
done
 
F_CONNECT $DBNAME $DEBUG_MODE || die $LINENO 10

# Get the number of partitions for this instance
F_RUN_SQL "select num_dbpartitions from sysibmadm.env_inst_info where inst_name = '${INSTNAME}'" "-xnq" 0 $DEBUG_MODE
RC=$? ; F_TEST_DB2_RETURNCODE $RC 0 $0 14 "Query error against sysibmadm.env_inst_info." || die $LINENO 11
num_parts=${SQLVAR1}

# Disable trap from here on, since we expect some non-zero return codes from DB2 commands.
# For example, there may not be a DCS directory.
trap " " ERR

print " 1. $(date) Capturing node, database and DCS directory settings.\n"
# NB There may be no remote databases catalogued in some environments and therefore no node directory (rc 4)
F_RUN_DB2CLP "list node directory"   4 ${CFG_DIR}/node.directory.${now} nocompress noattach $DEBUG_MODE 
RC=$? ; F_TEST_DB2_RETURNCODE $RC 4 $0 12 "Error running list node directory." || die $LINENO 12
# We always expect at least one local database.
F_RUN_DB2CLP "list database directory" 0 ${CFG_DIR}/database.directory.${now} nocompress noattach $DEBUG_MODE 
RC=$? ; F_TEST_DB2_RETURNCODE $RC 0 $0 13 "Error running list database directory." || die $LINENO 13
# A DCS directory is only required for catalogued OS/390, iSeries and VSE/VM host databases; rc 4 if it does not exist.
F_RUN_DB2CLP "list dcs directory" 4 ${CFG_DIR}/dcs.directory.${now} nocompress noattach $DEBUG_MODE 
RC=$? ; F_TEST_DB2_RETURNCODE $RC 4 $0 14 "Error running list database directory." || die $LINENO 14

print " 2. $(date) Capturing database manager, database, CLI and DAS configuration settings.\n"
F_RUN_DB2CLP "select name, value, value_flags, deferred_value, deferred_value_flags, datatype \
              from sysibmadm.dbmcfg order by name for fetch only" 0 ${CFG_DIR}/dbm.cfg.${now} nocompress noattach $DEBUG_MODE 
RC=$? ; F_TEST_DB2_RETURNCODE $RC 0 $0 15 "Query error against sysibmadm.dbmcfg." || die $LINENO 15
F_RUN_DB2CLP "select name, value, value_flags, deferred_value, deferred_value_flags, datatype, dbpartitionnum \
              from sysibmadm.dbcfg order by name for fetch only" 0 ${CFG_DIR}/db.cfg.${now} nocompress noattach $DEBUG_MODE 
RC=$? ; F_TEST_DB2_RETURNCODE $RC 0 $0 16 "Query error against sysibmadm.dbcfg." || die $LINENO 16
F_RUN_DB2CLP "get cli configuration" 0 ${CFG_DIR}/cli.cfg.${now} nocompress noattach $DEBUG_MODE
RC=$? ; F_TEST_DB2_RETURNCODE $RC 0 $0 17 "Error gettng the CLI cfg." || die $LINENO 17
F_RUN_DB2CLP "get admin configuration" 0 ${CFG_DIR}/admin.cfg.${now} nocompress noattach $DEBUG_MODE
RC=$? ; F_TEST_DB2_RETURNCODE $RC 0 $0 18 "Error gettng the admin cfg." || die $LINENO 18

print " 3. $(date) Capturing registry variable settings.\n"
F_RUN_DB2_SYSTEM_COMMAND "db2set" " -all" ${CFG_DIR}/db2set.${now} 0 nocompress $DEBUG_MODE
RC=$? ; F_TEST_DB2_RETURNCODE $RC 0 $0 19 "db2set command error." || die $LINENO 19

print " 4. $(date) Capturing audit settings.\n"
F_RUN_DB2_SYSTEM_COMMAND "db2audit" " describe" ${CFG_DIR}/db2audit.cfg.${now} 0 nocompress $DEBUG_MODE
RC=$? ; F_TEST_DB2_RETURNCODE $RC 0 $0 20 "db2audit command error." || die $LINENO 20

print " 5. $(date) Capturing db2look output.\n"
# db2look prevents variable expansion on the command line
typeset PARMS=" -d $DBNAME -e -x -l -o ${CFG_DIR}/db2look.${now}"
F_RUN_DB2_SYSTEM_COMMAND db2look "${PARMS}" "/dev/null" 0 nocompress $DEBUG_MODE
RC=$? ; F_TEST_DB2_RETURNCODE $RC 0 $0 21 "db2look command error." || die $LINENO 21

print " 6. $(date) Capturing db2level and db2pd output.\n"
F_RUN_DB2_SYSTEM_COMMAND "db2level" " " ${CFG_DIR}/db2level.${now} 0 nocompress $DEBUG_MODE
RC=$? ; F_TEST_DB2_RETURNCODE $RC 0 $0 22 "db2look command error." || die $LINENO 22
F_RUN_DB2_SYSTEM_COMMAND "db2pd" " -everything" ${CFG_DIR}/db2pd.${now} 0 compress $DEBUG_MODE
RC=$? ; F_TEST_DB2_RETURNCODE $RC 0 $0 23 "db2look command error." || die $LINENO 23

print " 7. $(date) Capturing list tablespaces and backup and archive log history.\n"
if   (( num_parts == 1 ))
then F_RUN_DB2CLP "list tablespaces show detail" 0 ${CFG_DIR}/tablespaces.${now} nocompress noattach $DEBUG_MODE
     RC=$? ; F_TEST_DB2_RETURNCODE $RC 0 $0 24 "List tablespaces error." || die $LINENO 24
     # On active systems, allow for rc 2 SQL2155W  Changes have been made to the recovery history file since the open scan was issued
	 F_RUN_DB2CLP "list history archive log all for database $DBNAME" 2 ${CFG_DIR}/archivehistory.${now} nocompress noattach $DEBUG_MODE
     RC=$? ; F_TEST_DB2_RETURNCODE $RC 2 $0 25 "List history archive log error." || die $LINENO 25
	 F_RUN_DB2CLP "list history backup all for database $DBNAME"  2 ${CFG_DIR}/backuphistory.${now} nocompress noattach $DEBUG_MODE
     RC=$? ; F_TEST_DB2_RETURNCODE $RC 2 $0 26 "List history backup error." || die $LINENO 26
else
	 db2_all "\"||db2 connect to $DBNAME; db2 list tablespaces show detail > ${CFG_DIR}/${DBNAME}.tablespaces.p##;db2 connect reset"	 
     RC=$? ; F_TEST_DB2_RETURNCODE $RC 0 $0 27 "List tablespaces error (db2_all)." || die $LINENO 27
     F_CONNECT $DBNAME $DEBUG_MODE || die $LINENO 28
     # On active systems, allow for rc 2 SQL2155W  Changes have been made to the recovery history file since the open scan was issued
	 db2_all "\"|| db2 list history archive log all for database $DBNAME >  ${CFG_DIR}/archivehistory.${DBNAME}.p##.${now}"  
	 RC=$? ; F_TEST_DB2_RETURNCODE $RC 2 $0 28 "List history archive log error (db2_all)." || die $LINENO 28
	 db2_all "\"|| db2 list history backup all for database $DBNAME >  ${CFG_DIR}/${DBNAME}.p##.backuphistory.${now}"  
     RC=$? ; F_TEST_DB2_RETURNCODE $RC 2 $0 29 "List history backup error (db2_all)." || die $LINENO 29
fi

print " 8. $(date) Capturing assorted flat files.\n"
crontab -l >  ${CFG_DIR}/cron.${now}
RC=$?; (( RC > 2 )) && die $LINENO 30
F_BACKUP_FILE $INSTHOME/.profile ${CFG_DIR}/${INSTNAME}.profile.${now} $DEBUG_MODE
RC=$? ; F_TEST_RETURNCODE $RC 0 $DEBUG_MODE || die $LINENO 31
F_BACKUP_FILE $INSTHOME/sqllib/userprofile ${CFG_DIR}/userprofile.${now} $DEBUG_MODE
RC=$? ; F_TEST_RETURNCODE $RC 0 $DEBUG_MODE || die $LINENO 32
F_BACKUP_FILE /etc/services ${CFG_DIR}/etc.services.${now} $DEBUG_MODE
RC=$? ; F_TEST_RETURNCODE $RC 0 $DEBUG_MODE || die $LINENO 33
F_BACKUP_FILE /etc/hosts ${CFG_DIR}/etc.hosts.${now} $DEBUG_MODE
RC=$? ; F_TEST_RETURNCODE $RC 0 $DEBUG_MODE || die $LINENO 34
for  file in $INSTHOME/.ssh/known_hosts $INSTHOME/.rhosts
do
	if   [[ -r $file ]]
	then base=$(basename $file)
		 F_BACKUP_FILE $file ${CFG_DIR}/${base}.${now}
	fi
done  

F_TERMINATE $DEBUG_MODE  

# Restore stdout and stderr to terminal
F_RESET_OUTPUT 
cat $LOG
exit 0
