#!/bin/ksh93
#set tabstop=4 to display correctly in vi
#
#-------------------------------------------------------------------------------
# File Name:    Lower_HWM.ksh (ksh93)
#				Based on existing copyleft code at website http://www.sustainablesoftware.net - please respect these terms.
#
# Purpose:      Lower the high water mark and reclaim space for all V9.7 automatic storage tablespaces.
#				Only bother if the HWM is 10% or more higher than the used pages.
#              
# Usage:        Lower_HWM.ksh $1 instance_name $2 database_name 
#               Assumes runner is the instance owner.
# 
# Out of scope: pre-V9.7 databases; tablespaces not under automatic storage.
#
# File dependencies:
#               ${INSTHOME}/sqllib/db2profile
#               ${INSTHOME}/sqllib/userprofile 
#               ${INSTHOME}/FUNCTIONS            - the userprofile exports this as $FPATH.
#				${INSTHOME}/MAINTENANCE          - the userprofile exports this as $MAINTENANCE_DIR.
#				${INSTHOME}/MAINTENANCE/log      - the userprofile exports this as $MAINTENANCE_LOGS.
#				${INSTHOME}/MAINTENANCE/log/work - the userprofile exports this as $MAINTENANCE_WORK.
#				${INSTHOME}/MAINTENANCE/lower_hwm.sql } sic
#				${INSTHOME}/MAINTENANCE/report_hwm.sql}
#
# File dependencies for partitioned databases:
#               None - this script operates on all partitions.
#
# Calls:		${MAINTENANCE}/Exit.ksh on error	
#
# Notes:        1. Read A_SETUP_README in this directory for local site customisations.
#               2. http://publib.boulder.ibm.com/infocenter/db2luw/v9r7/topic/com.ibm.db2.luw.admin.dbobj.doc/doc/t0055375.html
#                  " When you reduce the size of an automatic storage table space, the database manager attempts to lower the high water mark for the table space 
#					 AND (my emphasis) reduce the size of the table space containers. 
#					 In attempting to lower the high water mark, the database manager might drop empty containers and might move used extents to free space 
#					 nearer the beginning of the table space. 
#					 Next, containers are re-sized such that total amount of space in the table space is equal to or slightly greater than the high water mark."
#					 So, for automatic storage, we get two steps in 1: lower the HWM, then reduce, all through ALTER TABLESPACE ... REDUCE MAX. Neat!
#
# Date:          05.08.2010 and enhanced for publication on the web, 20.03.2011
# Modified       30.05.2012 Re-integrate into a complete maintenance suite for DB2 9.7  
#
# Author:       Alex Levy t/a http://www.sustainablesoftware.net
# Credits:		
# Date:         May 2012.           
#-------------------------------------------------------------------------------

# Functions in $FPATH never exit, they only return.
# We can either test each return code or use a trap. If we do use a trap, and we wish to test the return code first,
# then the test must be part of the same statement, e.g. F_CONNECT $DBNAME $DEBUG_MODE || die.
# The trap statement and the 2 inline functions 'FatalError' and 'die' can handle the exit for all error conditions.
# The 'die' function is called on any error detected by the program logic.
# The FatalError function is triggered by any non-zero signal, including the exit code from 'die'.

trap FatalError ERR

function FatalError {
# Single point of exit on any error condition.
# This relies on a signal trap. ERR is any non-zero exit status and is Korn shell only. 
# See 'man trap' for further detail.
# Can also call this function by 'kill -s ERR $$ 2>/dev/null'

RC=$? 
if   (( RC == 99 ))
then # error has already been trapped by function 'die', do nothing more except exit
	 ${MAINTENANCE_DIR}/Exit.ksh 99	
else
     print ERR signal received
	 if   [[ $LOG == "NOLOG" || ! -f $LOG || ! -s $LOG ]]
	 then F_MESSAGE "$script Ends with error(s), see stdout above." PLAIN I +x
	 else F_RESET_OUTPUT
     	  cat $LOG
     	  F_MESSAGE "$script ends with error(s). Please review log file $LOG . " PLAIN I +x | tee -a $LOG
     	  F_MESSAGE "The log file is displayed above for ease of reference. " PLAIN I +x | tee -a $LOG
	 fi
	 ${MAINTENANCE_DIR}/Exit.ksh 4	
fi
}

function die {
# Single point of exit on any error condition caught by the program logic.
# Restore stdout and stderr if required, before flowing to FatalError.

LOG=${LOG:=NOLOG} # use $LOG if set, otherwise assign a value of NOLOG to $LOG.
LINE=${1:=unknown}
LOCATION=${2:=unknown}

if   [[ $LOG == "NOLOG" || ! -f $LOG || ! -s $LOG ]]
then F_MESSAGE "$script Ends with error(s) at line $LINE, location $LOCATION; see stdout above." PLAIN I +x
else F_RESET_OUTPUT
     cat $LOG
     F_MESSAGE "$script ends with error(s) at line $LINE, location ${LOCATION}. Please review log file $LOG . " PLAIN I +x | tee -a $LOG
     F_MESSAGE "The log file is displayed above for ease of reference. " PLAIN I +x | tee -a $LOG
fi

# trigger FatalError
exit 99
} 

#-------------------------------------------------------------------------------
# Declare and test variables.
#-------------------------------------------------------------------------------

set -u								# assert all variables are defined and set.	 
typeset script=$(basename $0 .ksh)  # Strip off path and '.ksh' extension.
DEBUG_MODE="+x"                     # Default value - disable debugging.
typeset -i RC=0						# return code from function calls and o/s
typeset -i counter=999			
now=$(date +%Y%m%d%H%M%S)              
typeset LOG=NOLOG               	# set later

# Check the call
(( $# == 2 )) || F_ERROR_MESSAGE " Usage! I said:  $0 <instance_name> <database_name>" $DEBUG_MODE   
(( $# == 2 )) || die $LINENO 1 
typeset -l  INSTNAME=$1
typeset -l  DBNAME=$2
if   [[ $INSTNAME == $USER ]]
then :
else
     F_ERROR_MESSAGE "$0 : The runner must be the DB2 instance owner. Exiting ..." $DEBUG_MODE
     die $LINENO 2
fi

# Assert all required files and directories are in place and are accessible or executable;
# the first function call tests for $INSTHOME and the db2profile and executes db2profile.
F_SETUP_DB2_ENVIRONMENT $INSTHOME $INSTNAME $DBNAME $DEBUG_MODE 			|| die $LINENO 3
for dir in 	$FPATH 					\
			$MAINTENANCE_DIR 		\
			$MAINTENANCE_LOGS 		\
			$MAINTENANCE_WORK 		
do
	F_DIR_CHECK $dir $DEBUG_MODE || die $LINENO 4
done
for file in ${MAINTENANCE_DIR}/report_hwm.sql ${MAINTENANCE_DIR}/lower_hwm.sql
do
	F_READ_TEST $file $DEBUG_MODE || die $LINENO 5
done

# With these basic checks out of the way, direct all output to a timestamped log file.
LOG=${MAINTENANCE_LOGS}/${script}.${DBNAME}.${now}.log
F_INIT_OUTPUT_FILE $INSTNAME $LOG $DEBUG_MODE || die $LINENO 6
export LOG				 
F_REDIRECT_OUTPUT $LOG $DEBUG_MODE || die $LINENO 7

# Check no offline or online backup is running before connecting.
# This script could run concurrently against an online backup, but that would suggest a scheduling error. 
db2 -v attach to $INSTNAME || die $LINENO 8
while (( counter > 0 ))
do
    counter=$(db2 list utilities | grep -ic offline) || : #suppress trap of grep return code 1 if no match found
    (( counter > 0 )) && F_MESSAGE "Backup(s) are running." PLAIN I $DEBUG_MODE
    (( counter > 0 )) && sleep 300
done

export DB2NODE=0 # only meaningful in DPF
F_TEST_DBNAME  $DBNAME $DEBUG_MODE && F_CONNECT $DBNAME $DEBUG_MODE || die $LINENO 9

# Disable trap from here on, since we want explicit return code checks for DB2 commands and SQL queries. 
trap " " ERR

# Get the number of partitions for this instance
F_RUN_SQL "select num_dbpartitions from sysibmadm.env_inst_info where inst_name = '${INSTNAME}'" "-xnq" 0 $DEBUG_MODE
RC=$? ; F_TEST_DB2_RETURNCODE $RC 0 $0 14 "Query error against sysibmadm.env_inst_info." || die $LINENO 10
num_parts=${SQLVAR1}
 
# Report tablespace utilisation to $LOG
F_RUN_DB2FILE ${MAINTENANCE_DIR}/report_hwm.sql "-tvf" 0 NONE  nocompress $DEBUG_MODE
RC=$?; (( RC > 0 )) && F_MESSAGE "SQL runtime error, please review this job log." PLAIN I $DEBUG_MODE && die $LINENO 11
# Generate script to lower HWMs
F_RUN_DB2FILE ${MAINTENANCE_DIR}/lower_hwm.sql "-txf" 1  ${MAINTENANCE_WORK}/${script}_lower_hwm.clp.${now} nocompress $DEBUG_MODE
RC=$?
case $RC in
     0) # run the generated script
		if (( num_parts == 1 ))
		then F_RUN_DB2FILE ${MAINTENANCE_WORK}/${script}_lower_hwm.clp.${now}  "-tvf" 0 NONE  nocompress $DEBUG_MODE
		else
			 # Haven't tested function library or this script against a DPF installation yet
			 db2_all "\"|| db2 connect to $DBNAME; db2 -tvf ${MAINTENANCE_WORK}/${script}_lower_hwm.clp.${now}; db2 terminate"
		fi
		RC=$?; (( RC > 0 )) && F_MESSAGE "CLP runtime error, please review this job log." PLAIN I $DEBUG_MODE && die $LINENO 12
		print "Tablespace utilisation after lowering high water marks:"
		F_RUN_DB2FILE ${MAINTENANCE_DIR}/report_hwm.sql "-tvf" 0 NONE  nocompress $DEBUG_MODE
		RC=$?; (( RC > 0 )) && F_MESSAGE "SQL runtime error, please review this job log." PLAIN I $DEBUG_MODE && die $LINENO 13
		;;
	1) 	F_MESSAGE "Your tablespaces are in good shape at database ${DBNAME}! No high water marks need to be lowered today." PLAIN I $DEBUG_MODE
		;;
	*)	F_MESSAGE "SQL runtime error, please review this job log." PLAIN I $DEBUG_MODE && die $LINENO 14
		;;
esac

F_TERMINATE $DEBUG_MODE  

# Restore stdout and stderr to terminal
F_RESET_OUTPUT 
cat $LOG
exit $RC
