#!/bin/ksh93
#set tabstop=4 to display correctly in vi
#
#-------------------------------------------------------------------------------
# File Name:    Rebind_Packages.ksh (ksh93)
#				Based on existing copyleft code at website http://www.sustainablesoftware.net - please respect these terms.
# Author:       Alex Levy t/a http://www.sustainablesoftware.net
#
# Purpose:      Rebind all packages in the database.
#               Run after STATS/STATSCHK.ksh to take advantage of updated catalog statistics.
# 
# Usage:        Rebind_Packages.ksh instance_name database_name 
#               Assumes, but does not enforce, runner is the instance owner.
#
# File dependencies:
#               ${INSTHOME}/sqllib/db2profile
#               ${INSTHOME}/sqllib/userprofile 
#               ${INSTHOME}/FUNCTIONS   (the userprofile exports $FPATH).
#				${INSTHOME}/MAINTENANCE (the userprofile exports $MAINTENANCE_DIR).
#				${INSTHOME}/MAINTENANCE/log  (the userprofile exports $MAINTENANCE_LOGS).
#				${INSTHOME}/MAINTENANCE/log/work (the userprofile exports $MAINTENANCE_WORK).
#
# Calls:		${MAINTENANCE}/Exit.ksh on error	
#
# Notes:        http://publib.boulder.ibm.com/infocenter/db2luw/v9r7/topic/com.ibm.db2.luw.admin.cmd.doc/doc/r0001995.html
#
# Date:         June 2012.           
#-------------------------------------------------------------------------------

# Functions in $FPATH never exit, they only return.
# We can either test each return code or use a trap. If we do use a trap, and we wish to test the return code first,
# then the test must be part of the same statement, e.g. F_CONNECT $DBNAME $DEBUG_MODE || die.
# The trap statement and the 2 inline functions 'FatalError' and 'die' handle the exit for all error conditions.
# The 'die' function is called on any error detected by the program logic.
# The FatalError function is triggered by any non-zero signal, including the exit code from 'die'.

trap FatalError ERR

function FatalError {
# Single point of exit on any error condition.
# This relies on a signal trap. ERR is any non-zero exit status and is Korn shell only. 
# See 'man trap' for further detail.
# Can also call this function by 'kill -s ERR $$ 2>/dev/null'

RC=$? 
if   (( RC == 99 ))
then # error has already been trapped by function 'die', do nothing more except exit
	 ${MAINTENANCE_DIR}/Exit.ksh 99	
else
     print ERR signal received
	 if   [[ $LOG == "NOLOG" || ! -f $LOG || ! -s $LOG ]]
	 then F_MESSAGE "$script Ends with error(s), see stdout above." PLAIN I +x
	 else F_RESET_OUTPUT
     	  cat $LOG
     	  F_MESSAGE "$script ends with error(s). Please review log file $LOG . " PLAIN I +x | tee -a $LOG
     	  F_MESSAGE "The log file is displayed above for ease of reference. " PLAIN I +x | tee -a $LOG
	 fi
	 ${MAINTENANCE_DIR}/Exit.ksh 4	
fi
}

function die {
# Single point of exit on any error condition caught by the program logic.
# Restore stdout and stderr if required, before flowing to FatalError.

LOG=${LOG:=NOLOG} # use $LOG if set, otherwise assign a value of NOLOG to $LOG.
LINE=${1:=unknown}
LOCATION=${2:=unknown}

if   [[ $LOG == "NOLOG" || ! -f $LOG || ! -s $LOG ]]
then F_MESSAGE "$script Ends with error(s) at line $LINE, location $LOCATION; see stdout above." PLAIN I +x
else F_RESET_OUTPUT
     cat $LOG
     F_MESSAGE "$script ends with error(s). Please review log file $LOG . " PLAIN I +x | tee -a $LOG
     F_MESSAGE "The log file is displayed above for ease of reference. " PLAIN I +x | tee -a $LOG
fi

# trigger FatalError
exit 99
}

#-------------------------------------------------------------------------------
# Declare and test variables.
#-------------------------------------------------------------------------------

set -u								# assert all variables are defined and set.	 
typeset script=$(basename $0 .ksh)  # Strip off path and '.ksh' extension.
DEBUG_MODE="+x"                     # Default value - disable debugging.
typeset -i RC=0						# return code from function calls and o/s
typeset -i count=999				
now=$(date +%Y%m%d%H%M%S)              
typeset LOG=NOLOG               	# reset later

# Check the call
(( $# == 2 )) || F_ERROR_MESSAGE " Usage! I said:  $0 <instance_name> <database_name>" $DEBUG_MODE   
(( $# == 2 )) || die $LINENO 1 
typeset -l  INSTNAME=$1
typeset -u  DBNAME=$2
typeset     INSTHOME=~${INSTNAME}

# Assert all required files and directories are in place and are accessible or executable;
# the first function call tests for $INSTHOME and the db2profile and executes db2profile.
F_SETUP_DB2_ENVIRONMENT $INSTHOME $INSTNAME $DBNAME $DEBUG_MODE || die  $LINENO 2
F_DIR_CHECK $MAINTENANCE_DIR  $DEBUG_MODE   || die $LINENO 3
F_DIR_CHECK $MAINTENANCE_LOGS $DEBUG_MODE   || die $LINENO 4
F_DIR_CHECK $MAINTENANCE_WORK $DEBUG_MODE   || die $LINENO 5 

# check the database argument (is it catalogued as a local database?)
F_TEST_DBNAME $DBNAME $DEBUG_MODE || die $LINENO 6

# With these basic checks out of the way, direct all output to a timestamped log file.
LOG=${MAINTENANCE_LOGS}/${script}.${DBNAME}.${now}.log
F_INIT_OUTPUT_FILE $INSTNAME $LOG $DEBUG_MODE || die $LINENO 7
export LOG				 
F_REDIRECT_OUTPUT $LOG $DEBUG_MODE || die $LINENO 8

export DB2NODE=0 # only meaningful in DPF

# check no backup is running before connecting
db2 -v attach to $INSTNAME || die $LINENO 9
while (( count > 0 ))
do
    count=$(db2 list utilities | grep -ic backup | grep $DBNAME ) || : #suppress Unix trap of return code 1 if no match found
    (( count > 0 )) && F_MESSAGE "Backup(s) are running." PLAIN I $DEBUG_MODE
    (( count > 0 )) && sleep 300
done

# The db2rbind itself command does not require an existing database connection, but the query does.
F_CONNECT $DBNAME $DEBUG_MODE || die $LINENO 10
F_RUN_SQL "select count(*) from syscat.packages where valid = 'N' " "-xnq" 0 $DEBUG_MODE || die $LINENO 11
count=${SQLVAR1}
F_MESSAGE "$0 : There were $count invalid packages before running db2rbind." PLAIN I $DEBUG_MODE

F_RUN_DB2_SYSTEM_COMMAND "db2rbind" " $DBNAME -l ${MAINTENANCE_LOGS}/${script}.${now} all" $LOG 0 nocompress $DEBUG_MODE || RC=$?
(( RC > 0 )) && cat ${MAINTENANCE_LOGS}/${script}.${now} && die $LINENO 12
F_TERMINATE $DEBUG_MODE  

# Restore stdout and stderr to terminal
F_RESET_OUTPUT 
cat ${MAINTENANCE_LOGS}/${script}.${now}
cat $LOG
exit 0
