# !/bin/ksh
# File Name:    STATSCHK.ksh
#
# Author:       Alex Levy t/a Sustainable Software Ltd. http://www.sustainablesoftware.net
#
# Synopsis:     This utility creates a list of tables to RUNSTATS, and optionally carries out the RUNSTATS. 
#               By default, it runs the db2pd utility and lists ALL tables in permanent tablespaces with a positive Update Delete Insert (UDI) counter.
#               You can configure the candidate list so that it only considers tables in an optional list of tablespaces.
#               To do this, add the tablespaceid of each such tablespace on a separate line in the statschk_search_tbspaces file.
#               To consider all permanent tablespaces (i.e those not used for temporary data), just leave that file empty.
#               You can also specify an optional list of tablespaces whose tables must always be RUNSTATSd, regardless of UDI counts.
#               To do this, add the tablespaceid of each such tablespace on a separate line in the statschk_include_tbspaces file.
#               The assumption here is that related objects are grouped in the same tablespaces.
#				This last file is delivered with tablespace 0 - syscatspace - as the assumption is that the catalog tables always benefit from current stats. 
#				and the runtime is low.			
#
#               There is a final option to invoke RUNSTATS for each identified table.
#               Set the command line argument $3 to STATS to request this feature, otherwise set to NOSTATS.
#               The program will use a statistics profile, if one exists.  
#               Review the F_RUNSTATS function options for tables without an existing statistics profile, if you run with the STATS option.
#
#				The program produces a number of timestamped work files as well as a log file. These are left behind for problem determination.
# 				The $MAINTENANCE_DIR/PruneFiles.ksh script will housekeep all files if scheduled.
#
# Notes:        See the A_STATS_README file for a technical overview of the options for identifying RUNSTATS candidates.
#
# Usage:        STATSCHHK.ksh $1 $2 $3
#
# Parameters:   $1  instance_name 
#               $2  database_name (name, not alias - db2pd is particular!)
#				$3  statistics collection option: either "STATS" or "NOSTATS"
#
# Dependencies: $2 database is activated
#               ${INSTHOME}/sqllib/db2profile
#               ${INSTHOME}/sqllib/userprofile 
#               ${INSTHOME}/FUNCTIONS   (the userprofile exports $FPATH).
#				${INSTHOME}/MAINTENANCE (the userprofile exports $MAINTENANCE_DIR).
#				${INSTHOME}/MAINTENANCE/STATS - directory containing this script plus
#				${INSTHOME}/MAINTENANCE/STATS/statschk.awkin1 -  awk script file #1
#				${INSTHOME}/MAINTENANCE/STATS/statschk.awkin2 -  awk script file #2
#				${INSTHOME}/MAINTENANCE/STATS/statschk_search_tbspaces  : tablespaces whose table UDI counters are always checked - leave empty to consider all permanent tablespaces.
#				${INSTHOME}/MAINTENANCE/STATS/statschk_include_tbspaces : tablespaces whose tables and indexes are always RUNSTATS'd
#				${INSTHOME}/MAINTENANCE/log      (the userprofile exports $MAINTENANCE_LOGS).
#				${INSTHOME}/MAINTENANCE/log/work (the userprofile exports $MAINTENANCE_WORK).
#           
# Assumptions:   - Runs as the instance owner ;
#                - environment variable $INSTHOME (set by $INSTHOME/sqllib/db2profile) is exposed.
#
# Date:          Jan 2010
# 
# AMENDMENT HISTORY
# =================
# August 2011	Convert to ksh93.
#				Add DPF support (-dbp 0,1 and -full switches).
# June 2012		Integrate into a complete maintenance suite and retest at 9.7
# Sept 2012		Add commented-out logic for statistical views at tail of script. This is by call to a separate script, STATSVIEWS.ksh
#				You may likey prefer to schedule that script separately - see the 2 readme files.
# 
#---------------------------------------------------------------------------------------

# Functions in $FPATH never exit, they only return.
# We can either test each return code or use a trap. If we do use a trap, and we wish to test the return code first,
# then the test must be part of the same statement, e.g. F_CONNECT $DBNAME $DEBUG_MODE || die.
# The trap statement and the 2 inline functions 'FatalError' and 'die' handle the exit for all error conditions.
# The 'die' function is called on any error detected by the program logic.
# The FatalError function is triggered by any non-zero signal, including the exit code from 'die'.

trap FatalError ERR

function FatalError {
# Single point of exit on any error condition.
# This relies on a signal trap. ERR is any non-zero exit status and is Korn shell only. 
# See 'man trap' for further detail.
# Can also call this function by 'kill -s ERR $$ 2>/dev/null'

RC=$? 
if   (( RC == 99 ))
then # error has already been trapped by function 'die', do nothing more except exit
	 ${MAINTENANCE_DIR}/Exit.ksh 99	
else
     print ERR signal received
	 if   [[ $LOG == "NOLOG" || ! -f $LOG || ! -s $LOG ]]
	 then F_MESSAGE "$script Ends with error(s), see stdout above." PLAIN I +x
	 else F_RESET_OUTPUT
     	  cat $LOG
     	  F_MESSAGE "$script ends with error(s). Please review log file $LOG . " PLAIN I +x | tee -a $LOG
     	  F_MESSAGE "The log file is displayed above for ease of reference. " PLAIN I +x | tee -a $LOG
	 fi
	 ${MAINTENANCE_DIR}/Exit.ksh 4	
fi
}

function die {
# Single point of exit on any error condition caught by the program logic.
# Restore stdout and stderr if required, before flowing to FatalError.

LOG=${LOG:=NOLOG} # use $LOG if set, otherwise assign a value of NOLOG to $LOG.
LINE=${1:=unknown}
LOCATION=${2:=unknown}

if   [[ $LOG == "NOLOG" || ! -f $LOG || ! -s $LOG ]]
then F_MESSAGE "$script Ends with error(s) at line $LINE, location $LOCATION; see stdout above." PLAIN I +x
else F_RESET_OUTPUT
     cat $LOG
     F_MESSAGE "$script ends with error(s). Please review log file $LOG . " PLAIN I +x | tee -a $LOG
     F_MESSAGE "The log file is displayed above for ease of reference. " PLAIN I +x | tee -a $LOG
fi

# trigger FatalError
exit 99
}

#-------------------------------------------------------------------------------
# Declare and test variables.
#-------------------------------------------------------------------------------

set -u								# assert all variables are defined and set.	 
typeset script=$(basename $0 .ksh)  # Strip off path and '.ksh' extension.
DEBUG_MODE="+x"                     # Default value - disable debugging.
typeset -i RC=0						# return code from function calls and o/s
typeset -i num_partitions=-1		# reset later
typeset -i dbpartnum=-1     		# reset later
typeset -i count=999				
now=$(date +%Y%m%d%H%M%S)              
typeset LOG=NOLOG               	# reset later

# Check the call
(( $# == 3 )) || F_ERROR_MESSAGE " Usage! I said:  $0 <instance_name> <database_name> <STATS|NOSTATS>" $DEBUG_MODE   
(( $# == 3 )) || die $LINENO 1 
typeset -l  INSTNAME=$1
typeset -u  DBNAME=$2
typeset     INSTHOME=~${INSTNAME}
typeset -u  STATS_OPTION=$3
case ${STATS_OPTION} in
	 "STATS"|"NOSTATS") :
		;;
	 *) F_ERROR_MESSAGE " Usage! I said:  $0 <instance_name> <database_name> <STATS|NOSTATS>" $DEBUG_MODE 
        die $LINENO 2
		;;
esac

# Assert all required files and directories are in place and are accessible or executable;
# the first function call tests for $INSTHOME and the db2profile and executes db2profile.
F_SETUP_DB2_ENVIRONMENT $INSTHOME $INSTNAME $DBNAME $DEBUG_MODE || die  $LINENO 3
F_DIR_CHECK $MAINTENANCE_DIR  $DEBUG_MODE   || die $LINENO 4
F_DIR_CHECK $MAINTENANCE_LOGS $DEBUG_MODE   || die $LINENO 5
F_DIR_CHECK $MAINTENANCE_WORK $DEBUG_MODE   || die $LINENO 6 
F_DIR_CHECK $MAINTENANCE_DIR/STATS $DEBUG_MODE   || die $LINENO 7 

# the next 2 files can legitimately be empty, but must exist and be readable
for file in statschk_search_tbspaces statschk_include_tbspaces 
do
	F_EXIST_TEST $MAINTENANCE_DIR/STATS/$file $DEBUG_MODE || die $LINENO 8
done

for file in statschk.awkin1 statschk.awkin2
do
	F_READ_TEST $MAINTENANCE_DIR/STATS/$file $DEBUG_MODE || die $LINENO 8B
done

# check the database argument (is it catalogued as a local database?)
F_TEST_DBNAME $DBNAME $DEBUG_MODE || die $LINENO 9

# With these basic checks out of the way, direct all output to a timestamped log file.
LOG=${MAINTENANCE_LOGS}/${script}.${DBNAME}.${now}.log
F_INIT_OUTPUT_FILE $INSTNAME $LOG $DEBUG_MODE || die $LINENO 10
export LOG				 
F_REDIRECT_OUTPUT $LOG $DEBUG_MODE || die $LINENO 11

export DB2NODE=0 # only meaningful in DPF

# check no backup is running 
db2 -v attach to $INSTNAME || die $LINENO 12
while (( count > 0 ))
do
    count=$(db2 list utilities | grep -ic backup | grep $DBNAME ) || : #suppress Unix trap of return code 1 if no match found
    (( count > 0 )) && F_MESSAGE "Backup(s) are running." PLAIN I $DEBUG_MODE
    (( count > 0 )) && sleep 300
done
 
# Connection test - db2pd -tcbstats by itself does not require a connection, but catalog searches do.
F_CONNECT $DBNAME $DEBUG_MODE || die $LINENO 13

# Get the number of partitions for this instance
F_RUN_SQL "select num_dbpartitions from sysibmadm.env_inst_info where inst_name = '${INSTNAME}'" "-xnq" 0 $DEBUG_MODE
RC=$? ; F_TEST_DB2_RETURNCODE $RC 0 $0 14 "Query error against sysibmadm.env_inst_info." || die $LINENO 14
num_partitions=${SQLVAR1}
# set the default database partition argument for db2pd:
# - single partition instances -> dbp=0
# - multi-partition: select any database partition from the partition group on which the tablespace is distributed.
#	 We will check the database partition group in a DPF environment for each tablespace further down in the code.
#	 (It would be too simple to assume 0 is always the catalog partition, partition 1 is always reserved for small reference data that is replicated to remaining non-catalog partitions
#     and that fact tables are distributed across partitions 2 - n, and therefore UDI activity will always be reflected on partition 2.)
(( num_partitions == 1 )) && dbpstring=" -dbp 0 " || dbpstring=" -dbp "

# main logic here
# Disable trap from here on, since we want explicit return code tests on DB2 commands.
trap " " ERR

# See whether the caller wants to limit the statistics check to just some tablespaces;
# if not, select all regular and large tablespaces with permanent data (no temporary or user temporary tablespaces).

> ${MAINTENANCE_WORK}/${script}.statschk_search_tbspaces.${now}
if [[ -s $MAINTENANCE_DIR/STATS/statschk_search_tbspaces ]]
then
   	# file is populated, copy to work directory
   	cp -p $MAINTENANCE_DIR/STATS/statschk_search_tbspaces ${MAINTENANCE_WORK}/${script}.statschk_search_tbspaces.${now} 
else
	F_RUN_SQL "select tbspaceid from syscat.tablespaces where  datatype in ('A','L') order by tbspaceid for fetch only" "-x" 0 $DEBUG_MODE 
	RC=$? ; (( RC > 0 )) && F_MESSAGE "Query error against syscat.tablespaces return code $RC, see ${MAINTENANCE_WORK}/F_RUN_SQL.out." PLAIN I $DEBUG_MODE && die $LINENO 15
    cp -p  ${MAINTENANCE_WORK}/F_RUN_SQL.out ${MAINTENANCE_WORK}/${script}.statschk_search_tbspaces.${now}
fi

# The awkin scripts comment the logic for parsing db2pd output.
> ${MAINTENANCE_WORK}/${script}.tablist.${now} 
for tbspaceid in $(cat ${MAINTENANCE_WORK}/${script}.statschk_search_tbspaces.${now} )
do
	if   (( num_partitions == 1 ))
	then
		 eval db2pd -tcbstats TbspaceID=${tbspaceid} -db ${DBNAME} ${dbpstring} -o ${MAINTENANCE_WORK}/${script}.db2pd.out.${tbspaceid}.${now} 1> /dev/null # suppress informational messages
         RC=$? 
	else
		 # supply any database partition in this tablespace's nodegroup as an argument to db2pd;
		 # this assumes more or less even distribution across the partitions for each table - optimistic but not unreasonable.
		 F_RUN_SQL "select min(d.dbpartitionnum) from syscat.dbpartitiongroupdef d inner join syscat.tablespaces t on t.dbpgname = d.dbpgname where t.tbspaceid = $tbspaceid" "-xnq" 0 $DEBUG_MODE
         RC=$? ; F_TEST_DB2_RETURNCODE $RC 0 $0 16 "Query error against syscat.dbpartitiongroupdef." || die $LINENO 16
		 dbpartnum=${SQLVAR1}
		 eval db2pd -tcbstats TbspaceID=${tbspaceid} -db ${DBNAME} ${dbpstring} $dbpartnum -o ${MAINTENANCE_WORK}/${script}.db2pd.out.${tbspaceid}.${now} 1> /dev/null 
         RC=$? 
	fi
	# this return code check is of limited use, since db2pd can still produce RC=0 for bad calls:
    F_TEST_DB2_RETURNCODE $RC 0 $0 17 "$0 : db2pd call error on tcbstats for tablespace $tbspaceid - please review ${MAINTENANCE_WORK}/${script}.db2pd.out.${tbspaceid}.${now}" || die $LINENO 17
	# reset trap for o/s commands
	trap FatalError ERR
    awk -f ${MAINTENANCE_DIR}/STATS/statschk.awkin1 ${MAINTENANCE_WORK}/${script}.db2pd.out.${tbspaceid}.${now} >> ${MAINTENANCE_WORK}/${script}.awkout1.${now}
    awk -f ${MAINTENANCE_DIR}/STATS/statschk.awkin2 ${MAINTENANCE_WORK}/${script}.db2pd.out.${tbspaceid}.${now} >> ${MAINTENANCE_WORK}/${script}.awkout2.${now}
	trap " " ERR
done

# Sort both outputs on join field (table address), if both are populated:
if [[ -s ${MAINTENANCE_WORK}/${script}.awkout1.${now} && -s ${MAINTENANCE_WORK}/${script}.awkout2.${now} ]]
then
	trap FatalError ERR
	sort ${MAINTENANCE_WORK}/${script}.awkout1.${now} > ${MAINTENANCE_WORK}/${script}.awkout1.sorted.${now}
 	sort ${MAINTENANCE_WORK}/${script}.awkout2.${now} > ${MAINTENANCE_WORK}/${script}.awkout2.sorted.${now}
 	# Join on table address to obtain the initial list of tablespaceids and tableids for RUNSTATS:
 	join -o 1.2 1.3 -1 1  ${MAINTENANCE_WORK}/${script}.awkout1.sorted.${now} ${MAINTENANCE_WORK}/${script}.awkout2.sorted.${now} | sort -u > ${MAINTENANCE_WORK}/${script}.ids.${now}
	trap " " ERR
	while read tbspid tabid
	do
		db2 -x "select distinct rtrim(d.tabschema)||'.'||d.tabname 			\
				from syscat.datapartitions d inner join syscat.tables t 	\
					on d.tabschema = t.tabschema and d.tabname = t.tabname 	\
				where d.tbspaceid = $tbspid									\
				and   t.tableid = $tabid"	>> ${MAINTENANCE_WORK}/${script}.tablist.${now} 
	done < ${MAINTENANCE_WORK}/${script}.ids.${now}
    F_TEST_DB2_RETURNCODE $RC 1 $0 18 "$0 : SQL select error - please review ${MAINTENANCE_WORK}/${script}.tablist.${now} " || die $LINENO 18
else
	F_MESSAGE "No tables in the tablespace ids listed in ${MAINTENANCE_WORK}/${script}.statschk_search_tbspaces.${now} require RUNSTATS." PLAIN I $DEBUG_MODE
fi
  
# Add any tables that will always be RUNSTATS'D regardless 
if [[ -s ${MAINTENANCE_DIR}/STATS/statschk_include_tbspaces ]]
then
   # file has size > 0
   for tbspaceid in $(cat ${MAINTENANCE_DIR}/STATS/statschk_include_tbspaces)
   do
	   F_RUN_SQL "select distinct rtrim(tabschema)||'.'||tabname from syscat.datapartitions where  tbspaceid = $tbspaceid order  by 1 for fetch only" "-x" 0 $DEBUG_MODE 
	   F_TEST_DB2_RETURNCODE $RC 1 $0 19 "$0 SQL query error on syscat.datapartitions, tablespace $tbspaceid - please review ${MAINTENANCE_WORK}/${script}.tablist.${now}" || die $LINENO 19
   	   cat ${MAINTENANCE_WORK}/F_RUN_SQL.out >> ${MAINTENANCE_WORK}/${script}.tablist.${now}
   done 
   # The search and include files could overlap, so remove duplicates:
   sort -u ${MAINTENANCE_WORK}/${script}.tablist.${now} > ${MAINTENANCE_WORK}/${script}.runstats_tablist.${now}
else
   cp -p ${MAINTENANCE_WORK}/${script}.tablist.${now} ${MAINTENANCE_WORK}/${script}.runstats_tablist.${now}
fi

if [[ -s ${MAINTENANCE_WORK}/${script}.runstats_tablist.${now} ]]
then F_MESSAGE "Candidate tables for RUNSTATS are at ${MAINTENANCE_WORK}/${script}.runstats_tablist.${now}" PLAIN I $DEBUG_MODE
else F_MESSAGE "No tables require statistics updates." PLAIN I $DEBUG_MODE
fi

# RUNSTATS on identified tables, if requested;
if [[ $STATS_OPTION == "STATS" && -s ${MAINTENANCE_WORK}/${script}.runstats_tablist.${now} ]]
then
	F_MESSAGE "$0 :Updating catalog statistics ..." PLAIN I $DEBUG_MODE
    for table in $(cat ${MAINTENANCE_WORK}/${script}.runstats_tablist.${now} | grep -v SYSBOOT)
    do
        F_RUNSTATS_TABLE $table $DEBUG_MODE
        RC=$?
		# Allow for return code 2 warning messages suh as SQL2314W
        F_TEST_DB2_RETURNCODE $RC 2 $0 20 "$0 : Runstats error against table ${table}. Please review log." || die $LINENO 20 
    done
fi

# Unconditionally update stats for statistical views; comment out if you have no statistical views or wish to schedule separately.
#$MAINTENANCE_DIR/STATS/STATSVIEWS.ksh $INSTNAME $DBNAME
#RC=$? ; F_TEST_DB2_RETURNCODE $RC 2 $0 21 "$0 : Runstats error for statistical views. Please review log directory." || die $LINENO 21

F_TERMINATE $DEBUG_MODE  

# Restore stdout and stderr to terminal
F_RESET_OUTPUT 
cat $LOG
exit 0
