# !/bin/ksh
# File Name:    STATSVIEWS.ksh
#
# Author:       Alex Levy t/a Sustainable Software Ltd. http://www.sustainablesoftware.net
#
# Synopsis:     Unconditionally RUNSTATS statistical views. 
#               This is an extension to STATSCHK.ksh which you should run first, to RUNSTATS base tables.
#
#				The program produces a number of timestamped work files as well as a log file. These are left behind for problem determination.
# 				The $MAINTENANCE_DIR/PruneFiles.ksh script will housekeep all files if scheduled.
#
# Notes:        See the A_STATS_README file. 
#				You have the option to run this as a standalone script, or by call from STATSCHK.ksh
#				The code as supplied assumes it is a standalone script.
#				It therefore has its own $LOG logfile.
#				STATSCHK.ksh exports its own $LOG variable. 
#				To run by call from STATSCHK.ksh, simply comment out these 6 lines in the body:
#				typeset LOG=NOLOG                  # reset later
#				LOG=${MAINTENANCE_LOGS}/${script}.${DBNAME}.${now}.log
#				F_INIT_OUTPUT_FILE $INSTNAME $LOG $DEBUG_MODE || die $LINENO 10
#				export LOG
#				F_REDIRECT_OUTPUT $LOG $DEBUG_MODE || die $LINENO 11
#				F_RESET_OUTPUT
#				cat $LOG
#
# Usage:        STATSVIEWS.ksh $1 $2 
#
# Parameters:   $1  instance_name 
#               $2  database_name (name, not alias - db2pd is particular!)
#
# Dependencies: $2 database is activated
#               ${INSTHOME}/sqllib/db2profile
#               ${INSTHOME}/sqllib/userprofile 
#               ${INSTHOME}/FUNCTIONS   			(the userprofile exports $FPATH).
#				${INSTHOME}/MAINTENANCE 			(the userprofile exports $MAINTENANCE_DIR).
#				${INSTHOME}/MAINTENANCE/STATS 		- directory containing this script 
#				${INSTHOME}/MAINTENANCE/log      	(the userprofile exports $MAINTENANCE_LOGS).
#				${INSTHOME}/MAINTENANCE/log/work 	(the userprofile exports $MAINTENANCE_WORK).
#           
# Assumptions:   - Runs as the instance owner ;
#                - environment variable $INSTHOME (set by $INSTHOME/sqllib/db2profile) is exposed.
#
# Date:          Sept 2012
# 
# AMENDMENT HISTORY
# =================
#
#				
# 
#---------------------------------------------------------------------------------------

# Functions in $FPATH never exit, they only return.
# We can either test each return code or use a trap. If we do use a trap, and we wish to test the return code first,
# then the test must be part of the same statement, e.g. F_CONNECT $DBNAME $DEBUG_MODE || die.
# The trap statement and the 2 inline functions 'FatalError' and 'die' handle the exit for all error conditions.
# The 'die' function is called on any error detected by the program logic.
# The FatalError function is triggered by any non-zero signal, including the exit code from 'die'.

trap FatalError ERR

function FatalError {
# Single point of exit on any error condition.
# This relies on a signal trap. ERR is any non-zero exit status and is Korn shell only. 
# See 'man trap' for further detail.
# Can also call this function by 'kill -s ERR $$ 2>/dev/null'

RC=$? 
if   (( RC == 99 ))
then # error has already been trapped by function 'die', do nothing more except exit
	 ${MAINTENANCE_DIR}/Exit.ksh 99	
else
     print ERR signal received
	 if   [[ $LOG == "NOLOG" || ! -f $LOG || ! -s $LOG ]]
	 then F_MESSAGE "$script Ends with error(s), see stdout above." PLAIN I +x
	 else F_RESET_OUTPUT
     	  cat $LOG
     	  F_MESSAGE "$script ends with error(s). Please review log file $LOG . " PLAIN I +x | tee -a $LOG
     	  F_MESSAGE "The log file is displayed above for ease of reference. " PLAIN I +x | tee -a $LOG
	 fi
	 ${MAINTENANCE_DIR}/Exit.ksh 4	
fi
}

function die {
# Single point of exit on any error condition caught by the program logic.
# Restore stdout and stderr if required, before flowing to FatalError.

LOG=${LOG:=NOLOG} # use $LOG if set, otherwise assign a value of NOLOG to $LOG.
LINE=${1:=unknown}
LOCATION=${2:=unknown}

if   [[ $LOG == "NOLOG" || ! -f $LOG || ! -s $LOG ]]
then F_MESSAGE "$script Ends with error(s) at line $LINE, location $LOCATION; see stdout above." PLAIN I +x
else F_RESET_OUTPUT
     cat $LOG
     F_MESSAGE "$script ends with error(s). Please review log file $LOG . " PLAIN I +x | tee -a $LOG
     F_MESSAGE "The log file is displayed above for ease of reference. " PLAIN I +x | tee -a $LOG
fi

# trigger FatalError
exit 99
}

#-------------------------------------------------------------------------------
# Declare and test variables.
#-------------------------------------------------------------------------------

set -u								# assert all variables are defined and set.	 
typeset script=$(basename $0 .ksh)  # Strip off path and '.ksh' extension.
DEBUG_MODE="+x"                     # Default value - disable debugging.
typeset -i RC=0						# return code from function calls and o/s
typeset -i count=999				
now=$(date +%Y%m%d%H%M%S)              
typeset LOG=NOLOG               	# reset later

# Check the call
(( $# == 2 )) || F_ERROR_MESSAGE " Usage! I said:  $0 <instance_name> <database_name> " $DEBUG_MODE   
(( $# == 2 )) || die $LINENO 1 
typeset -l  INSTNAME=$1
typeset -u  DBNAME=$2
typeset     INSTHOME=~${INSTNAME}

# Assert all required files and directories are in place and are accessible or executable;
# the first function call tests for $INSTHOME and the db2profile and executes db2profile.
F_SETUP_DB2_ENVIRONMENT $INSTHOME $INSTNAME $DBNAME $DEBUG_MODE || die  $LINENO 2
F_DIR_CHECK $MAINTENANCE_DIR  $DEBUG_MODE   || die $LINENO 3
F_DIR_CHECK $MAINTENANCE_LOGS $DEBUG_MODE   || die $LINENO 4
F_DIR_CHECK $MAINTENANCE_WORK $DEBUG_MODE   || die $LINENO 5 
F_DIR_CHECK $MAINTENANCE_DIR/STATS $DEBUG_MODE   || die $LINENO 6 

# check the database argument (is it catalogued as a local database?)
F_TEST_DBNAME $DBNAME $DEBUG_MODE || die $LINENO 7

# With these basic checks out of the way, direct all output to a timestamped log file.
LOG=${MAINTENANCE_LOGS}/${script}.${DBNAME}.${now}.log
F_INIT_OUTPUT_FILE $INSTNAME $LOG $DEBUG_MODE || die $LINENO 8
export LOG				 
F_REDIRECT_OUTPUT $LOG $DEBUG_MODE || die $LINENO 9

# check no backup is running 
db2 -v attach to $INSTNAME || die $LINENO 10
while (( count > 0 ))
do
    count=$(db2 list utilities | grep -ic backup | grep $DBNAME ) || : #suppress Unix trap of return code 1 if no match found
    (( count > 0 )) && F_MESSAGE "Backup(s) are running." PLAIN I $DEBUG_MODE
    (( count > 0 )) && sleep 300
done
 
F_CONNECT $DBNAME $DEBUG_MODE || die $LINENO 11

# main logic here
# Disable trap from here on, since we want explicit return code tests on DB2 commands.
trap " " ERR

# Generate a file of RUNSTATS statements for statistical views; allow for no stats views (RC=1).
F_RUN_SQL	"select 'runstats on table '||rtrim(tabschema)||'.'||rtrim(tabname)||' ',\
            ' with distribution on all columns set profile;'   \
        	from   syscat.tables                               \
        	where  type  = 'V'                                 \
        	and    substr(property,13,1)='Y' "  "-x" 0 $DEBUG_MODE
RC=$?   
cp -p ${MAINTENANCE_WORK}/F_RUN_SQL.out ${MAINTENANCE_WORK}/${script}.${now} 
(( RC > 1  )) && F_MESSAGE "Query error against syscat.tables return code $RC, see ${MAINTENANCE_WORK}/${script}.${now}." PLAIN I $DEBUG_MODE && die $LINENO 12
(( RC == 1 )) && F_MESSAGE "Database $DBNAME has no statistical views. Nothing to do." PLAIN I $DEBUG_MODE
(( RC == 0 )) && F_RUN_DB2FILE ${MAINTENANCE_WORK}/${script}.${now} "-tvf" 2 NONE nocompress $DEBUG_MODE  # will output to $LOG
# Allow return code 2 for warning messages
RC=$? ; F_TEST_RETURNCODE $RC 2 $DEBUG_MODE || die $LINENO 13

F_TERMINATE $DEBUG_MODE  

# Restore stdout and stderr to terminal
F_RESET_OUTPUT 
cat $LOG
exit 0
